/*
 * Copyright 2015 Mat?j Kripner
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package cz.matejkripner;

import java.io.*;

/**
 * Main class which gets required information from the user and starts the {@link Program} class.
 *
 * @author Matej Kripner <kripnermatej@gmail.com>; google.com/+MatejKripner
 */
public class Main {

    public static void main(String[] args) {
        System.out.println("BRAINFUCK INTERPRET");
        System.out.println("author: Matej Kripner <kripnermatej@gmail.com>; please see google.com/+MatejKripner\n");
        try {
            new Main().launch(args);
        } catch (IOException e) {
            error("cannot read from file: " + e.getMessage());
        }
    }

    public void launch(String[] args) throws IOException {
        if (args.length < 2)
            error("Specify the source code file, the input file and the output file(optional) in that order");
        InputStream input = new FileInputStream(args[1]);
        PrintStream output = (args.length > 2) ? new PrintStream(new FileOutputStream(args[2]), false, "UTF-8") : System.out;
        Program program = new Program(readFile(args[0]));
        System.out.println("Program run\n------------------------------------");
        program.run(input, output);

        input.close();
        output.close();
    }

    private String readFile(String name) throws IOException {
        StringBuilder content = new StringBuilder();
        try (BufferedReader input = new BufferedReader(new FileReader(name))) {
            input.lines().forEach(content::append);
        }
        return content.toString();
    }

    public static void error(String msg) {
        System.err.println("ERROR: " + msg);
        System.exit(-1);
    }
}